searchNodes=[{"doc":"MuzakMuzak is a basic mutation testing library for Elixir. It is the limited, open source version of Muzak Pro. If you're not familiar with mutation testing, you can learn more about it in the guide here.","ref":"muzak.html","title":"Muzak","type":"extras"},{"doc":"To get started with mutation testing, first add muzak as a dependency in your mix.exs file and set the preferred_cli_env for muzak to test:defmodule MyApp.Mixfile do def project do [ # ... preferred_cli_env: [muzak: :test] ] end # ... defp deps do [ # ... {:muzak, &quot;~&gt; 1.0&quot;, only: :test} ] end endYou're now ready to get started!$ mix deps.get $ mix muzakMuzak will then randomly generate 25 mutations in your application and run your test suite against each of them. Each time you run mix muzak you will see different results.","ref":"muzak.html#getting-started","title":"Muzak - Getting started","type":"extras"},{"doc":"Configuration in Muzak is limited, but you can limit the mutations generated to a single file by using mix muzak --only path/to/file.ex. The limit of 25 mutations still exists, but this gives you some control over what is mutated so you can focus your testing on a single file.If you require additional configuration options, Muzak Pro will likely meet all those needs.","ref":"muzak.html#configuration","title":"Muzak - Configuration","type":"extras"},{"doc":"Muzak Pro is the full-featured, paid version of Muzak. It includes:No limit on the number of generated mutationsOver a dozen additional mutatorsgit integration to make mutation testing in CI blazing fastFar more configuration options at the command line and with a .muzak.exs fileExtensive documentation to help you get the most value possible out of mutation testingParallel execution by spawning multiple BEAM nodes (experimental)&quot;Analysis Mode&quot; to identify potentially low-value or duplicate tests in your test suite (coming soon)Enhanced reporting, including HTML reports (coming soon)Muzak Pro costs $29/month, and is available now.","ref":"muzak.html#muzak-pro","title":"Muzak - Muzak Pro","type":"extras"},{"doc":"Included MutatorsMuzak includes only three mutators - Constants.Numbers, Constants.Strings and Functions.Rename. While this is indeed rather limited, it should give some examples of the benefits of mutation testing.","ref":"mutators.html","title":"Included mutators","type":"extras"},{"doc":"Mutates any Integer or Float literals.Original Filedef do_math(num), do: num + 2 - 0.38Generated mutationsdef do_math(num), do: num + 327_237_729 - 0.38def do_math(num), do: num + 2 - 392_763.216_279","ref":"mutators.html#constants-numbers","title":"Included mutators - Constants.Numbers","type":"extras"},{"doc":"Mutates any string literals.Original Filedef append(str), do: str &lt;&gt; &quot; added on &quot; &lt;&gt; &quot; a string.&quot;Generated mutationsdef append(str), do: str &lt;&gt; &quot;random_string&quot; &lt;&gt; &quot; a string.&quot;def append(str), do: str &lt;&gt; &quot; added on &quot; &lt;&gt; &quot;random_string&quot;","ref":"mutators.html#constants-strings","title":"Included mutators - Constants.Strings","type":"extras"},{"doc":"Renames any function or macro definition.Original Filedef add_one(int), do: do_add_one(int) defp do_add_one(int), do: int + 1Generated mutationsdef random_function_name(int), do: do_add_one(int) defp do_add_one(int), do: int + 1def add_one(int), do: do_add_one(int) defp random_function_name(int), do: int + 1","ref":"mutators.html#functions-rename","title":"Included mutators - Functions.Rename","type":"extras"},{"doc":"What is Mutation Testing?Mutation testing is the process of programmatically introducing bugs to an application by mutating the application's source code and then running that application's test suite.","ref":"what_is_mutation_testing.html","title":"What is mutation testing?","type":"extras"},{"doc":"Mutation testing can serve several goals:Identifying untested code pathsIdentifying unused code pathsIdentifying tests that never failIdentifying tests that always fail together (duplicate coverage)Identifying tests that run slowly and rarely fail (what some consider &quot;low value&quot; tests)All of those benefits put together means that mutation testing can help us &quot;test our tests,&quot; both for any tests that might be missing, and to help us reduce time spent running unnecessary or low value tests.","ref":"what_is_mutation_testing.html#what-can-mutation-testing-do-for-me","title":"What is mutation testing? - What can mutation testing do for me?","type":"extras"},{"doc":"Yes, in its most basic form mutation testing is rather slow. You can make thousands of mutations in a reasonably sized application, and for each of those mutations you might potentially need to run your entire test suite. If there were no optimizations to how mutation testing was run, this would indeed take an unreasonably long time!To combat this, Muzak limits the number of mutations generated in each run to 25, and by default it will stop the test suite at the first failure for each mutation. This, combined with Elixir's easy ability to run asynchronous tests, makes mutation testing a relatively quick process, but at the cost of poor coverage for the application that's being tested.There are many better ways of reducing mutation testing run time that don't sacrifice nearly as much mutation coverage, though, and those options are all available in Muzak Pro.","ref":"what_is_mutation_testing.html#but-isn-t-mutation-testing-slow","title":"What is mutation testing? - But isn't mutation testing slow?","type":"extras"},{"doc":"Sure! Imagine that we have the below module:defmodule Authorization do def user_can_modify?(user, resource) do user.role in [:admin, :owner] or user.id in resource.member_ids end endAnd for that module, we have these tests:defmodule AuthorizationTest do use ExUnit.Case, async: true describe &quot;user_can_modify?/2&quot; do test &quot;returns true if the user is a member&quot; do user = %{id: 1, role: :reader} resource = %{member_ids: [1]} assert Authorization.user_can_modify?(user, resource) end test &quot;returns true if the user is an admin&quot; do user = %{id: 1, role: :admin} resource = %{member_ids: []} assert Authorization.user_can_modify?(user, resource) end test &quot;returns false if the user isn&#39;t an admin, owner or member&quot; do user = %{id: 1, role: :reader} resource = %{member_ids: []} refute Authorization.user_can_modify?(user, resource) end end endAll those tests will pass, and we also have 100% test coverage (if we're measuring by lines of code executed during our test suite). However, it is still possible to break that code in a way that won't trigger a failing test by applying the following mutation:# original user.role in [:admin, :owner] or user.id in resource.member_ids # mutation user.role in [:admin, :random_atom] or user.id in resource.member_idsTo resolve this, we would add a test that would catch this mutation:test &quot;returns true if the user is an owner&quot; do user = %{id: 1, role: :owner} resource = %{member_ids: []} assert Authorization.user_can_modify?(user, resource) endThis is just one very simple example of how mutation testing can help, but when it's used regularly in real applications it can make a huge difference.","ref":"what_is_mutation_testing.html#can-i-get-an-example","title":"What is mutation testing? - Can I get an example?","type":"extras"},{"doc":"No - and in fact, it may not even be possible! There are things that we can't test with a single automated test suite, such as code that's conditionally executed when running on a given operating system or in a given environment. There's also the case of &quot;equivalent mutants,&quot; where the mutation that's made satisfies all conditions in the application and therefore doesn't produce a failing test because the mutation is technically valid. This might mean that your application could be improved in some way, but this is often more of a hassle than it's worth.Of course test coverage needs vary from application to application, but 95% coverage is a reasonable goal that most applications can achieve.","ref":"what_is_mutation_testing.html#should-i-try-to-get-100-coverage","title":"What is mutation testing? - Should I try to get 100% coverage?","type":"extras"}]